# -*- coding: utf-8 -*-

import aioredis

from aiohttp_prometheus_monitoring.metrics.base import BaseMetric


class RedisMetric(BaseMetric):
    def __init__(self, host, port=6379, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.host = host
        self.port = port
        self.exc = aioredis.RedisError

    @staticmethod
    async def set_redis_value(host, port):
        conn = await aioredis.create_connection((host, port))
        response = await conn.execute('SET', 'test-redis', 'test-redis')

        return conn, response

    async def check(self):
        conn, response = await self.set_redis_value(self.host, self.port)
        conn.close()
        return response == b'OK'



# -*- coding: utf-8 -*-

import aioamqp

from aiohttp_prometheus_monitoring.metrics.base import BaseMetric


class AmqpMetric(BaseMetric):
    def __init__(self, user, password, host, port=5672, vhost='/', *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.url = 'amqp://{user}:{password}@{host}:{port}/{vhost}'.format(
            user=user,
            password=password,
            host=host,
            port=port,
            vhost=vhost
        )
        self.exc = aioamqp.exceptions.AioamqpException

    @staticmethod
    async def connect_amqp(url):
        transport, protocol = await aioamqp.from_url(url)
        return transport, protocol

    async def check(self):
        transport, protocol = await self.connect_amqp(self.url)
        is_connected = protocol is not None and protocol.state == aioamqp.protocol.OPEN and transport is not None

        if is_connected:
            await protocol.close()

        return is_connected




MONITORING = {
    'route_metrics': '/metrics',

    'metrics': [
        {
            'name': 'monitoring_redis',
            'description': 'Check redis connection',
            'module': 'aiohttp_prometheus_monitoring.metrics.redis.RedisMetric',
            'sleep_time': 60,
            'params': {
                'host': 'localhost',
                'port': '6379',
            }
        },
    ]
}



# -*- coding: utf-8 -*-

from importlib import import_module

from prometheus_client import Gauge
from prometheus_async.aio.web import server_stats

from aiohttp_prometheus_monitoring.utils import PeriodicTask
from aiohttp_prometheus_monitoring.exceptions import WrongConfiguration
from aiohttp_prometheus_monitoring.views import ping


def import_by_path(dotted_path):
    try:
        module_path, object_name = dotted_path.rsplit('.', 1)
    except ValueError:
        raise WrongConfiguration("{} doesn't look like a module path".format(dotted_path))
    try:
        module = import_module(module_path)
    except ImportError as e:
        raise WrongConfiguration("Error importing module {}: {}".format(module_path, e))
    try:
        attr = getattr(module, object_name)
    except AttributeError:
        raise WrongConfiguration('Module "%s" has no "%s" object' % (module_path, object_name))

    return attr


async def init_metrics(config):
    tasks = []
    if 'metrics' in config:
        try:
            for metric in config['metrics']:
                metric_class = import_by_path(metric['module'])

                kwargs = metric['params']
                kwargs['collector'] = Gauge(metric['name'], metric['description'], None)

                task = PeriodicTask(metric['sleep_time'], metric_class, **kwargs)
                await task.start()
                tasks.append(task)

        except KeyError as e:
            raise WrongConfiguration('Config key {} is mandatory'.format(str(e)))

    return tasks


def init_monitoring_routes(config, app):
    if 'route_ping' in config:
        app.router.add_route('GET', config['route_ping'], ping)

    if 'route_metrics' in config:
        app.router.add_route('GET', config['route_metrics'], server_stats)


async def stop_monitoring_tasks(app):
    for task in app.monitoring_tasks:
        await task.stop()


async def setup_monitoring(config, app=None):
    app.monitoring_tasks = await init_metrics(config)
    app.on_shutdown.append(stop_monitoring_tasks)
    init_monitoring_routes(config, app)



# -*- coding: utf-8 -*-

import asyncio
from contextlib import suppress


class PeriodicTask:
    def __init__(self, sleep_time, metric_class, *args, **kwargs):
        self.metric = metric_class(*args, **kwargs)
        self.sleep_time = sleep_time
        self.args = args
        self.kwargs = kwargs
        self.is_started = False
        self._task = None

    async def start(self):
        if not self.is_started:
            self.is_started = True
            self._task = asyncio.ensure_future(self._run())

    async def stop(self):
        if self.is_started:
            self.is_started = False
            self._task.cancel()
            with suppress(asyncio.CancelledError):
                await self._task

    async def _run(self):
        while True:
            await self.metric.run_check()
            await asyncio.sleep(self.sleep_time)



# -*- coding: utf-8 -*-

import asyncio
import aiohttp  # noqa

from aiohttp_prometheus_monitoring.metrics.base import BaseMetric


class HttpMetric(BaseMetric):
    def __init__(self, url, timeout=10, verify_ssl=True, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.url = url
        self.timeout = timeout
        self.verify_ssl = verify_ssl
        self.exc = (aiohttp.ClientError, asyncio.TimeoutError)

    @staticmethod
    async def http_get_status(url, timeout, verify_ssl):
        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=verify_ssl)) as session:
            async with session.get(url, timeout=timeout) as response:
                return response.status

    async def check(self):
        status = await self.http_get_status(self.url, self.timeout, self.verify_ssl)
        return status == 200



# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__package__)


class BaseMetric:
    collector = None
    exc = ()

    def __init__(self, collector):
        self.collector = collector

    async def check(self) -> bool:
        raise NotImplemented

    async def run_check(self):
        try:
            ok = await self.check()
            self.collector.set(int(ok))
        except self.exc as e:
            logger.exception(e)
            self.collector.set(0)
        except Exception as e:
            logger.exception(e)
            self.collector.set(-1)



import aiomysql

from pymysql.err import DatabaseError

from aiohttp_prometheus_monitoring.metrics.base import BaseMetric


class MySQLMetric(BaseMetric):
    def __init__(self, database, user, password, host, port=3006, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.connection_params = {
            'db': database,
            'user': user,
            'password': password,
            'host': host,
            'port': port
        }
        self.exc = DatabaseError

    @staticmethod
    async def select_one(connection_params):
        async with aiomysql.create_pool(**connection_params) as pool:
            async with pool.get() as conn:
                async with conn.cursor() as cur:
                    await cur.execute('SELECT 1;')
                    result = await cur.fetchall()
                    return result

    async def check(self):
        response = await self.select_one(self.connection_params)
        return response == ((1,),)



#!/usr/bin/env python

import asyncio
import argparse

from aiohttp import web
from aiohttp_prometheus_monitoring import setup_monitoring

import settings


def create_app(loop):
    app = web.Application()
    loop.run_until_complete(setup_monitoring(settings.MONITORING, app))
    return app


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Example Http Worker', add_help=True)
    parser.add_argument('-p', '--port', required=False, default=9000, type=int, help='port for http')
    parser.add_argument('--host', required=False, default='0.0.0.0', type=str, help='host for http')
    args = parser.parse_args()

    loop = asyncio.get_event_loop()
    web.run_app(create_app(loop), host=args.host, port=args.port)



#!/usr/bin/env python

import os

from setuptools import setup, find_packages


ROOT_DIR = os.path.abspath(os.path.dirname(__file__))
VERSION_FILE = os.path.join(ROOT_DIR, 'VERSION')
BUILD_NUMBER = os.environ.get('BUILD_NUMBER')


def read_file(file_path):
    return open(file_path).read().strip()


__version__ = read_file(os.path.join(ROOT_DIR, 'VERSION'))


if BUILD_NUMBER:
    full_version = '%s.%s' % (__version__, BUILD_NUMBER)
    f = open(VERSION_FILE, 'w')
    try:
        f.writelines([full_version])
    finally:
        f.close()


setup(
    name='aiohttp_prometheus_monitoring',
    version=read_file(os.path.join(ROOT_DIR, 'VERSION')),
    packages=find_packages(),
    long_description=read_file(os.path.join(ROOT_DIR, 'README.md')),
    include_package_data=True,
    author='Wargaming Team',
    install_requires=[
        'aiohttp',
        'prometheus_client>=0.0.21',
        'prometheus-async>=17.4.0',
    ],
    extras_require={
        'amqp': ['aioamqp>=0.9.0'],
        'postgres': ['aiopg'],
        'redis': ['aioredis'],
        'mysql': ['aiomysql'],
    },
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ]
)



# -*- coding: utf-8 -*-

import aiopg
from psycopg2 import DatabaseError

from aiohttp_prometheus_monitoring.metrics.base import BaseMetric


class PostgresMetric(BaseMetric):
    def __init__(self, database, user, password, host, port=5432, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.dsn = 'dbname={database} user={user} password={password} host={host} port={port}'.format(**{
            'database': database,
            'user': user,
            'password': password,
            'host': host,
            'port': port
        })
        self.exc = DatabaseError

    @staticmethod
    async def select_one(dsn):
        async with aiopg.create_pool(dsn) as pool:
            async with pool.acquire() as conn:
                async with conn.cursor() as cur:
                    await cur.execute('SELECT 1')
                    result = []
                    async for row in cur:
                        result.append(row)
                    return result

    async def check(self):
        response = await self.select_one(self.dsn)
        return response == [(1,)]
